import numpy as np
import scipy.stats as ss
import stan
import pickle
import arviz as az
import pandas as pd

pd.options.display.max_rows = 1000
pd.options.display.max_columns = 1000

stan_script = """
    data {
        int N;
        array[N] real X;
    }
    parameters {
        real mu;
        real<lower=0.0> sgm;
    }
    model {
        mu ~ normal(.0, 10.0);  
        sgm ~ exponential(0.1); 
        X ~ normal(mu, sgm);
    }
"""

 
def CalcMAPEst(samples, a = 0.05, n_points = 2000):
    """
        Calculatte a MAP estimate from a KDE graph on [Lp, Up]
        Lp and Up are 100*a/2 and 100(1-a/2) percentile points of samples
    """
    Lp, Up = np.percentile(samples, [100 * a/2, 100 * (1 - a/2)])  # import numpy as np
    coord = np.linspace(Lp, Up, n_points)
    est_pdf = ss.gaussian_kde(samples).pdf(coord)  # import scipy.stats as ss
    map_idx = np.argmax(est_pdf)
    MAP_Est = coord[map_idx]     
    return MAP_Est, est_pdf[map_idx]
 

N = 10

n_simu = 100
mu_means = []
mu_medians = []
mu_maps = []
sgm_means = []
sgm_medians = []
sgm_maps = []
ws = ''
for i in range(90):
    ws += ' '
for simu in range(n_simu):
    print(ws, f'simu = {simu}/{n_simu}       ', end = '\r')
    X = ss.norm.rvs(size = N)

    sm = stan.build(stan_script,data = {'N':len(X), 'X':X} )
    fit = sm.sample(num_samples=10000)

    mu_mean = np.mean(fit['mu'])
    mu_median = np.median(fit['mu'])
    mu_map = CalcMAPEst(fit['mu'])[0]
    mu_means.append(mu_mean)
    mu_medians.append(mu_median)
    mu_maps.append(mu_map)

    sgm_mean = np.mean(fit['sgm'])
    sgm_median = np.median(fit['sgm'])
    sgm_map = CalcMAPEst(fit['sgm'])[0]
    sgm_means.append(sgm_mean)
    sgm_medians.append(sgm_median)
    sgm_maps.append(sgm_map)

with open('temp.pkl', 'wb') as f:
    pickle.dump((mu_means, mu_medians, mu_maps,
                 sgm_means, sgm_medians, sgm_maps), f)
print('temp.pkl was saved.')
