import numpy as np
import scipy.stats as ss
import matplotlib.pyplot as plt
import seaborn as sb
import pickle
import arviz as az
import pandas as pd

 
def CalcMAPEst(samples, a = 0.05, n_points = 2000):
    """
        Calculatte a MAP estimate from a KDE graph on [Lp, Up]
        Lp and Up are 100*a/2 and 100(1-a/2) percentile points of samples
    """
    Lp, Up = np.percentile(samples, [100 * a/2, 100 * (1 - a/2)])  # import numpy as np
    coord = np.linspace(Lp, Up, n_points)
    est_pdf = ss.gaussian_kde(samples).pdf(coord)  # import scipy.stats as ss
    map_idx = np.argmax(est_pdf)
    MAP_Est = coord[map_idx]     
    return MAP_Est, est_pdf[map_idx]
 

with open('temp.pkl', 'rb') as f:
    (mu_means, mu_medians, mu_maps, sgm_means, sgm_medians, sgm_maps) = \
        pickle.load(f)


mu_means = np.array(mu_means)
mu_medians = np.array(mu_medians)
mu_maps = np.array(mu_maps)
rms_mu_means = np.mean((mu_means - 0)**2)
rms_mu_medians = np.mean((mu_medians - 0)**2)
rms_mu_maps = np.mean((mu_maps - 0)**2)
print('RMS(mu_mean) =', rms_mu_means)
print('RMS(mu_median) =', rms_mu_medians)    
print('RMS(mu_map) =', rms_mu_maps)

sgm_means = np.array(sgm_means)
sgm_medians = np.array(sgm_medians)
sgm_maps = np.array(sgm_maps)
rms_sgm_means = np.mean((sgm_means - 1)**2)
rms_sgm_medians = np.mean((sgm_medians - 1)**2)
rms_sgm_maps = np.mean((sgm_maps - 1)**2)
print('\nRMS(sgm_mean) =', rms_sgm_means)
print('RMS(sgm_median) =', rms_sgm_medians)    
print('RMS(sgm_map) =', rms_sgm_maps)
sb.kdeplot(mu_means, label = 'mean')
sb.kdeplot(mu_medians, label = 'median')
sb.kdeplot(mu_maps, label = 'map')
plt.plot([0, 0], [0, 1.2], c = 'r', ls = '--', label ='$\mu = 0$')
plt.title(f'RMSEs of estimates of $\mu$\n{rms_mu_means:.3f}(for means)' +
          f',  {rms_mu_medians:.3f}(for medians))' +
          f',  {rms_mu_maps:.3f}(for maps)', fontsize = 14)
plt.xlabel('$\hat\mu$', fontsize = 16)
plt.legend()
plt.tight_layout()
plt.savefig('FigMuHat.png')
plt.show()

sb.kdeplot(sgm_means, label = 'mean')
sb.kdeplot(sgm_medians, label = 'median')
sb.kdeplot(sgm_maps, label = 'map')
plt.plot([1, 1], [0, 1.4], c = 'r', ls = '--', label = '$\sigma = 1$')
plt.title(f'RMSEs of estimates of $\sigma$\n{rms_sgm_means:.3f}(for means)' +
          f',  {rms_sgm_medians:.3f}(for medians))' +
          f',  {rms_sgm_maps:.3f}(for maps)', fontsize = 14)
plt.xlabel('$\hat\sigma$', fontsize = 16)
plt.legend()
plt.savefig('FigSgmHat.png')
plt.show()

