import numpy as np
import scipy.stats as ss
import stan
import pickle
import arviz as az
import pandas as pd

pd.options.display.max_rows = 1000
pd.options.display.max_columns = 1000

stan_script = """
    data {
        int N;
        array[N] real X;
    }
    parameters {
        real mu;
        real<lower=0.0> sgm;
    }
    model {
        mu ~ normal(.0, 10.0);  
        sgm ~ exponential(0.1); 
        X ~ normal(mu, sgm);
    }
"""

N = 10
X = ss.norm.rvs(size = N)

sm = stan.build(stan_script,data = {'N':len(X), 'X':X} )
fit = sm.sample(num_samples=10000)   

inference_data = az.from_pystan(posterior = fit, posterior_model = sm)
smry = az.summary(inference_data)
print('Summary =\n',smry)

df = fit.to_frame()
with open('d_frame.pkl', 'wb') as f:
    pickle.dump(df, f)
